#!/bin/bash
#
# This script updates the database structure from the collectors 1.0.x to the
# new 1.1.0 collector.
#
# Author: Abilis <support@antek.it>
# Date:   20/07/2011
# Ver:    1.1.0-1


# --- CONSTANTS DEFINITIONS ---

# Script version
VER="1.1.0-1"

# The full path of configuration file
CONFFILE="/etc/collector/collector.conf"

# SQL file for updating db structure from 1.0.x to 1.1.0
STRCT_UPD_1_0_x_TO_1_1_0="sql/global_1.0.x_to_1.1.0.sql"

# SQL template file for updating collector tables from 1.0.x to 1.1.0
TBL_TEMPL_UPD_1_0_x_TO_1_1_0="sql/collector_table_1.0.x_to_1.1.0.sql.tpl"

# --- END CONSTANTS DEFINITIONS ---

echo
echo "#################################"
echo "#   Collector database update   #"
echo "#          Ver. ${VER}         #"
echo "#              ***              #"
echo "#    Abilis: www.abilis.net     #"
echo "#################################"
echo

# Check if CONFFILE exists
if [[ ! -e ${CONFFILE} ]]; then
  echo -e "\nERROR: File ${CONFFILE} not found\n"
  exit 1
fi

# Get full path of the update script
SCRIPT_FULL_PATH=`readlink -f $0`

# Obtain database info from the CONFFILE
MYSQLHOST=`grep MYSQLHOST ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLPORT=`grep MYSQLPORT ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLDB=`grep MYSQLDB ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLUSER=`grep MYSQLUSER ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLPASS=`grep MYSQLPASS ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLTABLES=`egrep ^MYSQLTABLE ${CONFFILE} | sort | uniq | awk -F ':' '{print $2}'`

# Check if required database fields are present
for keyword in MYSQLHOST MYSQLDB MYSQLUSER; do
  if [[ -z ${!keyword} ]]; then
    echo -e "\nERROR: ${keyword} is not specified in ${CONFFILE}!"
    echo -e "Please set the correct db credentials in ${CONFFILE}\n"
    exit 2
  fi
done

# Set default database port if not set into collector.conf
if [[ -z ${MYSQLPORT} ]]; then
  echo "INFO: MYSQLPORT is not set, assuming 3306"
  MYSQLPORT=3306
fi

# Assume no password if MYSQLPASS is not set
if [[ -z ${MYSQLPASS} ]]; then
  echo "INFO: MYSQLPASS is not set"
else
  MYSQLPASS="-p${MYSQLPASS}"
fi

# Ask the user if he wants to perform a database backup before update
choice=""
do_backup=0
while [[ ! ${choice} =~ ^[yn]$ ]]; do
  echo -e "\nDatabase backup is STRONGLY SUGGESTED!"
  echo -en "Do you want to backup your current database (y/n)? "
  read choice

  if [[ ${choice} == "y" ]]; then 
    echo -e "\nOk, DB backup will be made, a wisdom choice man!"

    backup_path_ok=0
    while [[ ${backup_path_ok} -eq 0 ]]; do
      echo -e "\nPlease choose the path where you want to save the backup"
      echo -en "(press enter for default: `dirname ${SCRIPT_FULL_PATH}`): "
      read backup_path
      
      if [[ -z ${backup_path} ]]; then
        backup_path=`dirname ${SCRIPT_FULL_PATH}`
      fi

      if [[ -d ${backup_path} ]]; then
        backup_path_ok=1
        do_backup=1
      else
        echo -e "\nThe path you specified is not valid. Please enter a valid one"
      fi
    done

  elif [[ ${choice} == "n" ]]; then 
    echo -e "\nOk, DB backup will NOT be made. :/"
  else 
    echo -e "Please choose either y or n" 
  fi

done

# Summary
echo
echo "###############"
echo "#   Summary   #"
echo "###############"
echo
echo -e "MySQL Host:\t${MYSQLHOST}"
echo -e "MySQL Port:\t${MYSQLPORT}"
echo -e "MySQL DB:\t${MYSQLDB}"
echo -e "MySQL User:\t${MYSQLUSER}"
echo
if [[ ${do_backup} == 1 ]]; then
  echo "DB Backup: YES"
  echo "DB Backup file: ${backup_path}/${MYSQLDB}.sql.gz"
else
  echo "DB Bckup: NO"
fi
echo

choice=""
while [[ ! ${choice} =~ (^yes$)|(^no$) ]]; do  
  echo -n "Do you want to continue (yes/no)? " 
  read choice 
  echo 

  if [[ ${choice} == "yes" ]]; then 
    echo -e "*** Beginning update ***\n" 
  elif [[ ${choice} == "no" ]]; then 
    echo -e "*** Update aborted! ***\n" 
    exit 1 
  else 
    echo -e "Please choose either yes or no" 
  fi 
done

# Check the existance of table version_info in current database
mysql ${MYSQLDB} -u${MYSQLUSER} -e 'DESCRIBE version_info' > /dev/null 2>&1
if [[ $? == 0 ]]; then
  # The version_info table exists, so no update should be performed
  echo "INFO: The database seems to be already up to date"
  echo
  echo "Nothing to do, bye!"
  echo
  exit 255
fi

# (eventually) backup current db
if [[ ${do_backup} == 1 ]]; then
  echo -n "Executing database backup... "
  mysqldump ${MYSQLDB} -u${MYSQLUSER} ${MYSQLPASS} | gzip -c > ${backup_path}/${MYSQLDB}.sql.gz
  
  if [[ $? -eq 0 ]]; then
    echo "Done!"
  else
    echo -e "\nERROR: Backup failed, aborting :(\n"
    exit 255
  fi

fi

# Begin updating the global database structure
echo -n "Updating global structure from 1.0.x to 1.1.0... "
mysql ${MYSQLDB} -u${MYSQLUSER} ${MYSQLPASS} < ${STRCT_UPD_1_0_x_TO_1_1_0}
if [[ $? -eq 0 ]]; then
  echo "Done!"
else
  echo -e "\nERROR: Update failed, sorry :(\n"
  exit 255
fi

# Begin updating the collector tables (which may be more than one)
echo "Updating collector tables from 1.0.x to 1.1.0:"
for table in ${MYSQLTABLES}; do
  echo -en "  Updating table \"$table\"... "
  sed "s/__TABLE_NAME__/${table}/g" ${TBL_TEMPL_UPD_1_0_x_TO_1_1_0} | mysql ${MYSQLDB} -u${MYSQLUSER} ${MYSQLPASS} 
  if [[ $? -eq 0 ]]; then
    echo "Done!"
  else
    echo -e "\nERROR: Update failed, sorry :(\n"
    exit 255
  fi
done
echo "All done!"

echo -e "\nDB Update succesfully completed\n"

