#include <errno.h>
#include <sys/types.h>
#include <mysql/mysql.h>
#include <mysql/errmsg.h>
#include <mysql/mysqld_error.h>
#include "md5.h"

#ifndef COLLECTOR_DEF_H
#define COLLECTOR_DEF_H

#define COLLECTOR_VERSION   "1.1.0"
#define DATABASE_VERSION    2
#define DATABASE_VERSION_TABLE	"version_info"

/* user defined parameters */
#define CONFIG_FILE	    "/etc/collector/collector.conf"
#define PID_FILE	    "/etc/collector/collector.pid"
#define QUEUE_FILE	    "/etc/collector/collector.queue"
#define QUEUE_MAX_PROCESS   10 /* max cached queries to process each time */
#define QUEUE_INTERVAL	    120 /* queue process interval in seconds */
#define KEEP_ALIVE	    60 /* keep alive interval in seconds */
#define RETRY_MESSAGE	    7
#define RETRY_DISABLED	    (-1)
#define POLLING_TIMEO_SEC   1
#define TIME_BUF_LEN	    32 /* formatted time length */
#define USE_PROCFS	    1
#define MYSQL_MAX_VLEN	    2560 /* max length of MySQL query values */
#define MYSQL_MAX_QLEN	    4096 /* max length of MySQL (total) query */
#define SESSION_END_TIMEOUT     10 /* Timeout (in sec) waiting for session
                                      end response */

/* configuration parameters */
#define MAX_PATH_LEN	    256
#define MAX_CONFVAL_LEN	    128
#define MAX_CLIENTID_LEN    32
#define ERROR_LEN	    256

/* packet format */
#define PACKET_HLEN_SIZE	2
#define PACKET_MAX_SIZE		2050
#define PACKET_DATA_POSITION	4
#define PACKET_ACNT_POSITION	4
#define AUTH_DATA_SIZE		16

/* message format */
#define PLAIN_MESSAGE	    0
#define ENCRYPTED_MESSAGE   1

/* mode codes */
#define RESERVED_MODE	    0
#define SIMPLE_MODE_1K	    1
#define TRANSACTION_MODE_1K 2
#define SIMPLE_MODE_2K	    3
#define TRANSACTION_MODE_2K 4

/* session protocol */
#define DUMMY_MESSAGE			    0
#define COLLECTOR_AUTHENTICATION_REQUEST    1
#define COLLECTOR_AUTHENTICATION_FAILURE    2
#define AGENT_AUTHENTICATION_REQUEST	    3
#define AGENT_AUTHENTICATION_FAILURE	    4
#define AVAILABLE_MODES_REQUEST		    5
#define AVAILABLE_MODES_INDICATE	    6
#define MODE_SETTING			    7 
#define MODE_NOT_AVAILABLE		    8
#define MODE_ACCEPTED			    9
#define MODE_MESSAGE			    10
#define TERMINATE_MODE			    11
#define TERMINATE_MODE_ACKNOWLEDGE	    12
#define TERMINATE_SESSION		    13
#define TERMINATE_SESSION_ACKNOWLEDGE	    14
#define INVALID_MESSAGE			    15

/* accounting protocol */
#define ACNT_IE_PORT_IN                 ( 0x01 )
#define ACNT_IE_CDI                     ( 0x02 )
#define ACNT_IE_SDI                     ( 0x03 )
#define ACNT_IE_CGI                     ( 0x04 )
#define ACNT_IE_SGI                     ( 0x05 )
#define ACNT_IE_EBIB_IN                 ( 0x06 )
#define ACNT_IE_SIDE_IN                 ( 0x07 )
#define ACNT_IE_TYPE_IN                 ( 0x08 )
#define ACNT_IE_PARENT_ID_IN            ( 0x09 )
#define ACNT_IE_USER_IN                 ( 0x0A )

#define ACNT_IE_PORT_OUT                ( 0x11 )
#define ACNT_IE_CDO                     ( 0x12 )
#define ACNT_IE_SDO                     ( 0x13 )
#define ACNT_IE_CGO                     ( 0x14 )
#define ACNT_IE_SGO                     ( 0x15 )
#define ACNT_IE_EBIB_OUT                ( 0x16 )
#define ACNT_IE_SIDE_OUT                ( 0x17 )
#define ACNT_IE_TYPE_OUT                ( 0x18 )
#define ACNT_IE_PARENT_ID_OUT           ( 0x19 )
#define ACNT_IE_USER_OUT                ( 0x1A )

#define ACNT_IE_CONNECTION_TYPE         ( 0x21 )
#define ACNT_IE_CAUSE                   ( 0x22 )
#define ACNT_IE_BEARER                  ( 0x23 )
#define ACNT_IE_INCOMING_DATE_TIME      ( 0x24 )
#define ACNT_IE_END_DATE_TIME           ( 0x25 )
#define ACNT_IE_CONNECTION_STATE        ( 0x26 )
#define ACNT_IE_CALL_LENGTH             ( 0x27 )
#define ACNT_IE_DISCONNECT_DIRECTION    ( 0x28 )

#define ACNT_IE_DISCONNECT_DATE_TIME    ( 0x29 )
#define ACNT_IE_CALL_RESULT             ( 0x2A )
#define ACNT_IE_CALL_DIRECTION          ( 0x2B )
#define ACNT_IE_CALL_ID                 ( 0x2C )

#define ACNT_IE_VOICE_COMPRESSION       ( 0x41 )
#define ACNT_IE_FAX_RELAY               ( 0x42 )
#define ACNT_IE_MODEM_RELAY             ( 0x43 )

#define ACNT_IE_NEW_VOICE_COMPRESSION   ( 0x44 )
#define ACNT_IE_NEW_FAX_RELAY           ( 0x45 )
#define ACNT_IE_NEW_MODEM_RELAY         ( 0x46 )
#define ACNT_IE_RESERVED_BANDWIDTH      ( 0x47 )
#define ACNT_IE_LOCAL_AC_SES_INFO       ( 0x48 )
#define ACNT_IE_REMOTE_AC_SES_INFO      ( 0x49 )

#define ACNT_IE_TC_AUDIO_CODEC_PROPERTIES (0x54)
#define ACNT_IE_TC_FAX_CODEC_PROPERTIES   (0x55)
#define ACNT_IE_TC_DATA_CODEC_PROPERTIES  (0x56)
#define ACNT_IE_TC_RESERVED_BANDWIDTH     (0x57)
#define ACNT_IE_TC_LOCAL_AC_SES_INFO      (0x58)
#define ACNT_IE_TC_REMOTE_AC_SES_INFO     (0x59)

#define ACNT_IE_LOST_RECORDS            ( 0x7F )

typedef enum {
	TERMINATE,
	TERMINATION_SENT,
	CLOSED,
	DISCONNECTED,
	CONNECTED,
	AUTHENTICATION_SENT,
	AUTHENTICATED_AGENT,
	MODES_SENT,
	MODE_NEGOTIATED,
	MODE_SET,
	RUNNING,
	ALIVE_RECV,
} fsm_t;

/* global vars and struct */
struct CfgClient {
    char id[MAX_CLIENTID_LEN];
    char agentHost[MAX_CONFVAL_LEN];
    int agentPort;
    char mysqlTable[MAX_CONFVAL_LEN];
    int status; /* client<=>agent socket */
    fsm_t fsm;
    char error[ERROR_LEN]; /* last error */
    int timeo; /* max time to send/recv a message */
    int alive; /* saved time for keep alive packets */
    int reconnect; /* saved time for last reconnection try */
};

struct QryQueue {
    long position;
    int length;
    int processed;
    int inserted;
    int skipped;
    int duplicate;
    int nextCheck;
};

struct CfgData {
    struct QryQueue queue;
    char uptime[32];
    char mysqlHost[MAX_CONFVAL_LEN];
    int mysqlPort;
    char mysqlUser[MAX_CONFVAL_LEN];
    char mysqlPass[MAX_CONFVAL_LEN];
    char mysqlDb[MAX_CONFVAL_LEN];
    int retry;
    long clients;
    MYSQL mysqlLink; /* client<=>MySQL connection object */
};

/* widely used functions */
void printError(const char *fmt, ...);
void printDebug(const char *fmt, ...);
void printDump(void *packet, int size);
void getHumanTime(char *tbuffer);

void setError(struct CfgClient *client, char *error);
void setInactive(struct CfgClient *client, char *error);
int isConfigured(struct CfgClient *client);
int isActive(struct CfgClient *client);

#define COLLECTOR_CONTINUE	0
#define COLLECTOR_HALT		1
void setShutdown(int action, int retval);

#endif
/* 
 * vim: softtabstop=4 shiftwidth=4 
 */


