/* session_protocol.c - session protocol client<=>agent
 *
 * Copyright (C) 2006-2007 Abilis
 * Copyright (C) 2006-2007 Antek srl
 *
 * Author: Elisa Manara <e@entropika.net>
 * Last update: Mon Dec 11 10:01:26 CET 2006
 *
 * This file is part of Abilis Collector for *nix.
 *
 * Abilis Collector is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Abilis Collector is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 *                                   
 * You should have received a copy of the GNU General Public License along                        
 * with Abilis Collector; see the file COPYING.  If not, write to the Free                        
 * Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. 
 */  

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <poll.h>
#include "define.h"

static int sendPacket(struct CfgClient *client, unsigned int cmd, unsigned int method,
               unsigned char *data, unsigned int datalen);
int recvPacket(struct CfgClient *client, unsigned int *cmd, unsigned int *method,
               unsigned char *data, unsigned int maxdatalen);
void accountingManager(struct CfgData *config, struct CfgClient *client, 
    unsigned char *data, unsigned int datalen);

static int tcpConnect(char *host, int port);


static int waitForSend(struct CfgClient *client) {
	struct pollfd psend = {
	    .fd = client->status,
	    .events = POLLOUT,
	};

	poll(&psend, 1, POLLING_TIMEO_SEC*1000);
	if (psend.revents & POLLOUT)
		return (0);

	return (1);
}

/* Create the socket and connect to host/port.
 * Return -1 in case of errors, socket descriptor otherwise.
 */
int tcpConnect(char *host, int port)
{
    int s;
    struct sockaddr_in sa;

    if ((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        printError("Creating socket: %s\n", strerror(errno));
        return -1;
    }
    sa.sin_family = AF_INET;
    sa.sin_port = htons(port);
    if (inet_aton(host, &sa.sin_addr) == 0) {
        struct hostent *he;

        he = gethostbyname(host);
        if (he == NULL) {
            printError("gethostbyname() Cannot resolve host %s - %s\n", 
	    	host, strerror(errno));
            close(s);
            return -1;
        }
        memcpy(&sa.sin_addr, he->h_addr, sizeof(struct in_addr));
    }
    if (connect(s, (struct sockaddr*)&sa, sizeof(sa)) == -1) {
        printDebug("connect(): %s\n", strerror(errno));
        close(s);
        return -1;
    }
    return s;
}

/* Send a packet to the Agent
 * 
 * s: file descriptor 
 * cmd: command code
 * method: method code
 * data: attached data
 * datalen: attached data length
 *
 * Return -1 if some error occurs, otherwise the length of send packet
 */
static int sendPacket(struct CfgClient *client, unsigned int cmd, unsigned int method,
               unsigned char *data, unsigned int datalen)
{
    int totlen = PACKET_DATA_POSITION + datalen;
    int nwritten;
    time_t now;
    unsigned char *p = malloc(totlen);

    if (!p) {
   	printError("sendPacket() malloc: %s\n", strerror(errno));
    	return -1; /* out of memory */
    }
    /* fill packet headers */
    p[0] = (totlen>>8) & 0xFF;
    p[1] = totlen&0xFF;
    p[2] = cmd&0xFF;
    p[3] = method&0xFF;

    /* attach data */
    memcpy(p+PACKET_DATA_POSITION, data, datalen);
    nwritten = write(client->status, p, totlen);
    free(p);

    if (nwritten != totlen) return -1;
    /* update keep alive time */
    now = time(NULL);
    client->alive = now + KEEP_ALIVE;

    return totlen;
}

/* Receive a packet and extract command, method, data.
 *
 * s: file descriptor
 * cmd: pointer for command code
 * method: pointer for method code
 * data: pointer for data
 * maxdatalen: max length of data buffer
 *
 * Return -1 if some errors occur, 0 for closed connection,
 * length of packet data otherwise
 */
int recvPacket(struct CfgClient *client, unsigned int *cmd, unsigned int *method,
               unsigned char *data, unsigned int maxdatalen)
{
    int nread, totlen, datalen;
    unsigned char hdr[PACKET_HLEN_SIZE];

    nread = read(client->status,hdr,PACKET_HLEN_SIZE);
    if (nread == 0) {
	/* remote side closed connection */
	return 0;
    }
    if (nread != PACKET_HLEN_SIZE) {
	printError("Read packet header: expected %d bytes, got %d\n", 
	    PACKET_HLEN_SIZE, nread);
    	return -1;
    }

    totlen = hdr[0]<<8 | hdr[1];
    if (totlen < PACKET_DATA_POSITION) {
	printError("Read packet length: expected > %d bytes, got %d\n", 
	    PACKET_DATA_POSITION, totlen);
	return -1; /* protocol error.. */
    } 

    //printDebug("Received packet totlen = %d\n", totlen);

    datalen = totlen - PACKET_HLEN_SIZE;

    if ((unsigned)datalen > maxdatalen) {
	printDebug("Packet data size %d, recv buffer %d\n", datalen, maxdatalen);
    	return -1; /* too little recv buffer! */
    }
    nread = read(client->status, data, datalen);
    //printDebug("Read bytes, nread: %i\n", nread);
    if (nread != datalen) {
	printError("Short read or EOF! Expected %d bytes, got %d\n",
		datalen, nread);
	return -1; /* short read or EOF! */
    }
    *cmd = data[0];
    *method = data[1];
    return datalen;
}

int connectToAgent(struct CfgClient *client, time_t now) {
    /* client not configured => return */
    if (!isConfigured(client))
	return (1);
    /* it's not time to reconnect yet */
    if (client->reconnect > now)
	return (1);

    client->status = tcpConnect(client->agentHost, client->agentPort);

    if (client->status == -1) {
	printError("[%s] Connection failed to host #%s#: %s\n",
	    client->id, client->agentHost, strerror(errno));
	setInactive(client, strerror(errno));
	return (-1);
    }

    printDebug("[%s] Connected to host %s\n",
	client->id, client->agentHost);
    return (0);
}

int sendAuthentication(struct CfgClient *client, char *userdata) {
    unsigned char p[3*AUTH_DATA_SIZE];
    unsigned char authdata[AUTH_DATA_SIZE];
    MD5_CTX md5;
    int len, i;

    /* random fill */
    for(i = 0; i < AUTH_DATA_SIZE; i++)
	authdata[i] = random()&0xFF; 

    MD5Init(&md5);
    MD5Update(&md5, authdata, AUTH_DATA_SIZE);
    MD5Final(&md5);

    /* prepare auth packet */
    memcpy(p,authdata,AUTH_DATA_SIZE);
    memcpy(p+AUTH_DATA_SIZE,md5.digest,AUTH_DATA_SIZE);
    memcpy(p+(2*AUTH_DATA_SIZE),userdata,AUTH_DATA_SIZE);

    if (waitForSend(client))
	return (1);

    /* send auth request */
    len = sendPacket(client,COLLECTOR_AUTHENTICATION_REQUEST,
	PLAIN_MESSAGE,p,(3*AUTH_DATA_SIZE));

    if (len < 0) {
	printError("[%s] Error sending authentication request\n",
	    client->id);
	setInactive(client, "Error sending authentication request");
	return (-1);
    }

    printDebug("[%s] Sent auth request: %d bytes written\n",
	client->id, len);
    return (0);
}

int checkAgentAuthentication(struct CfgClient *client, unsigned int cmd, unsigned int m,
    unsigned char *data, unsigned int len) {

    if (cmd != AGENT_AUTHENTICATION_REQUEST) {
	printError("[%s] Expected AGENT_AUTHENTICATION_REQUEST, got: %.2x\n",
	    client->id, cmd);
	setInactive(client, "Authentication failed");
	return (-1);
    };

    printDebug("[%s] Got AGENT_AUTHENTICATION_REQUEST from agent %s\n",
	client->id, client->agentHost);

    /* TODO check agent authentication */
    /* TODO AGENT_AUTHENTICATION_FAILURE */

    return (0);
}

int sendModesRequest(struct CfgClient *client) {
    int len;

    if (waitForSend(client))
	return (1);

    len = sendPacket(client, AVAILABLE_MODES_REQUEST, PLAIN_MESSAGE, NULL, 0);
    if (len < 0) {
	printError("[%s] Error sending AVAILABLE_MODES_REQUEST to agent %s\n",
	    client->id, client->agentHost);
	setInactive(client, "Error sending AVAILABLE_MODES_REQUEST to agent");
	return (-1);
    }

    return (0);
}

int checkSupportedModes(struct CfgClient *client, unsigned int cmd, unsigned int m,
    unsigned char *data, unsigned int len) {
    int i;

    if (cmd != AVAILABLE_MODES_INDICATE) {
	printError("AVAILABLE_MODES_INDICATE expected, got %i\n", cmd);
	setInactive(client, "Error receiving AVAILABLE_MODES_INDICATE from agent");
	return (-1);
    }

    for (i=PACKET_HLEN_SIZE; i<len; i++) 
	printDebug("[%s] AVAILABLE MODE from agent: %2x\n", 
	    client->id, data[i]&0xFF);
    return (0);
}

int setMode(struct CfgClient *client, int mode) {
    int len;
    unsigned char p[1];

    p[0] = mode&0xFF;

    if (waitForSend(client))
	return (1);

    /* set mode command */
    len = sendPacket(client, MODE_SETTING, PLAIN_MESSAGE, p, 1);
    if (len < 0)
	return (-1);

    return (0);
}

int checkModeAcceptance(struct CfgClient *client, unsigned int cmd, unsigned int m,
    unsigned char *data, unsigned int len) {

    if (cmd != MODE_ACCEPTED) {
	printError("[%s] Mode '%i' not accepted by agent %s\n",
	    client->id, SIMPLE_MODE_2K, client->agentHost);
	setInactive(client, "Mode not accepted by agent");
	return (-1);
    }

    printDebug("[%s] Set mode '%i': accepted\n",
	client->id, SIMPLE_MODE_2K);
    return (0);
}

int sendDummy(struct CfgClient *client)
{
    int len;

    if (waitForSend(client))
	return (1);

    len = sendPacket(client, DUMMY_MESSAGE, PLAIN_MESSAGE, NULL, 0);
    return len;
}

void sessionKeepAlive(struct CfgClient *client, time_t now)
{
    if(client->alive <= now) {
	sendDummy(client);
	client->alive = now + KEEP_ALIVE;
	printDebug("[%s] Keep alive sent to %s\n", 
	    client->id, client->agentHost);
    }
}

int sendTerminateSession(struct CfgClient *client, char *reason)
{
    int len;
    int rlen=strlen(reason);
    unsigned char *p;

    p = malloc(rlen+1);
    if (!p) {
   	printError("sendTerminateSession() malloc: %s\n", strerror(errno));
    	return -1; /* out of memory */
    }

    /* set reason length */
    p[0] = rlen&0xFF;

    /* attach data */
    memcpy(p+1, reason, rlen);

    if (waitForSend(client))
	return (1);

    len = sendPacket(client,TERMINATE_SESSION,PLAIN_MESSAGE,p,rlen+1);
    free(p);

    return (0);
}

int checkAgentTermination(struct CfgClient *client, unsigned int cmd, unsigned int m,
    unsigned char *data, unsigned int len) {

    if(cmd != TERMINATE_SESSION_ACKNOWLEDGE) {
	printError("TERMINATE_SESSION_ACKNOWLEDGE expected, got %i\n", cmd);
	return (-1);
    }

    return (0);
}

/* handle incoming packet messages and switch to
 * appropriate funtions
 */
int handlePacket(struct CfgData *config, struct CfgClient *client, 
    unsigned int cmd, unsigned int method, unsigned char *data, 
    unsigned int datalen) 
{
    switch(cmd) {
    /* TODO here ... other messages */
    case DUMMY_MESSAGE:
	client->fsm = ALIVE_RECV;
	break;
    case MODE_MESSAGE:
	accountingManager(config, client, data, datalen);
	break;
    case INVALID_MESSAGE:
	printError("[%s] Last sent message to %s is not valid\n",
	    client->id, client->agentHost);
	break;
    default:
        printDebug("[%s] Received packet from agent %s - cmd: %.2x, method: %.2x\n", 
		client->id, client->agentHost, cmd, method);
	break;
    }
    return 0;
}

/* 
 * vim: softtabstop=4 shiftwidth=4 
 */
